﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.IO;
using System.Xml;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;

namespace SaveCustomization
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {  
        public MainWindow()
        {
            InitializeComponent();

            Data = new ObservableCollection<Data>();

            for (int i = 0; i < 10; i++)
            {
                Data.Add(new Data { ID = i.ToString(), Complex = new ComplexData { Name = "Name " + i, Value = i * 3 }, Bool = "test"});//i % 2 == 0 });
            }

            DataContext = this;
        }
         
        public ObservableCollection<Data> Data { get; set; }
 
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            using (FileStream stream = new FileStream("data.xml", FileMode.Open, FileAccess.Read))
            {
                XmlDocument dock = new XmlDocument();
                dock.Load(stream);

                original.Text = PrintXML(dock.InnerXml);

            }

            using (FileStream stream = new FileStream("data.xml", FileMode.Open, FileAccess.Read))
            { 
                LoadCustomizationHelper.LoadCustomizations(xamDataGrid1, stream);

                modified.Text = xamDataGrid1.SaveCustomizations();
            } 
        }


        public String PrintXML(String XML)
        {
            String Result = "";

            MemoryStream mStream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(mStream, Encoding.Unicode);
            XmlDocument document = new XmlDocument();

            try
            {
                // Load the XmlDocument with the XML.
                document.LoadXml(XML);

                writer.Formatting = Formatting.Indented;

                // Write the XML into a formatting XmlTextWriter
                document.WriteContentTo(writer);
                writer.Flush();
                mStream.Flush();

                // Have to rewind the MemoryStream in order to read
                // its contents.
                mStream.Position = 0;

                // Read MemoryStream contents into a StreamReader.
                StreamReader sReader = new StreamReader(mStream);

                // Extract the text from the StreamReader.
                String FormattedXML = sReader.ReadToEnd();

                Result = FormattedXML;
            }
            catch (XmlException)
            {
            }

            mStream.Close();
            writer.Close();

            return Result;
        }
    }

    public static class LoadCustomizationHelper
    {
        #region LoadCustomizations
        public static void LoadCustomizations(XamDataGrid grid, string customizations)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                StreamWriter sw = new StreamWriter(ms);
                sw.Write(customizations);
                sw.Flush();
                ms.Position = 0;

                string xml = GetModifiedXml(ms, grid);

                grid.LoadCustomizations(xml);
            }
        }

        public static void LoadCustomizations(XamDataGrid grid, string customizations, out string error)
        {
            string errorLocal = "";
            using (MemoryStream ms = new MemoryStream())
            {
                StreamWriter sw = new StreamWriter(ms);
                sw.Write(customizations);
                sw.Flush();
                ms.Position = 0;

                string xml = GetModifiedXml(ms, grid);

                grid.LoadCustomizations(xml, out errorLocal);

                error = errorLocal;
            }
        }

        public static void LoadCustomizations(XamDataGrid grid, Stream stream)
        {
            string xml = GetModifiedXml(stream, grid);

            grid.LoadCustomizations(xml);
        }

        public static void LoadCustomizations(XamDataGrid grid, Stream stream, out string error)
        {
            string errorLocal = "";
            string xml = GetModifiedXml(stream, grid);

            grid.LoadCustomizations(xml, out errorLocal);

            error = errorLocal;
        }
        #endregion

        #region Private Methods
        private static string GetModifiedXml(Stream stream, XamDataGrid grid)
        {
            XmlDocument customizations = null;
            using (BufferedStream strm = new BufferedStream(stream))
            {
                customizations = new XmlDocument();
                customizations.Load(strm);
            }

            if (customizations != null)
            {
                XmlNode rootNode = customizations.SelectSingleNode("xamDataPresenter");
                XmlNode fls = rootNode.SelectSingleNode("fieldLayouts");

                foreach (XmlNode fl in fls.SelectNodes("fieldLayout"))
                {
                    XmlNode flds = fl.SelectSingleNode("fields");

                    string newFieldList = "";
                    List<XmlNode> fieldsToRemove = new List<XmlNode>();

                    foreach (XmlNode fld in flds.SelectNodes("field"))
                    {
                        if (!IsFieldPresent(grid, fl.Attributes["key"].Value, fld.Attributes["name"].Value))
                        {
                            fieldsToRemove.Add(fld);
                            continue;
                        }


                        XmlAttribute extendInfo = fld.Attributes["extendedInfo"];
                        if (extendInfo != null)
                        {
                            if (extendInfo.Value.Contains("unbound"))
                                newFieldList += extendInfo.Value + ", ";
                            else
                                fld.Attributes.Remove(extendInfo);
                        }
                        else
                        {
                            newFieldList += fld.Attributes["name"].Value + ", ";
                        }
                    }

                    foreach (XmlNode fld in fieldsToRemove)
                    {
                        flds.RemoveChild(fld);
                    }

                    fl.Attributes["fieldList"].Value = newFieldList.Substring(0, newFieldList.Length - 2);
                }

                return customizations.InnerXml;
            }

            return "";
        }

        private static bool IsFieldPresent(XamDataGrid grid, string flKey, string fldName)
        {

            FieldLayout layout = grid.FieldLayouts.Where(
                fl =>
                    (fl.Key != null && fl.Key.Equals(flKey))
                    || (fl.Key != null && (fl.Key as Type).Name.Equals(flKey))).FirstOrDefault();

            if (layout != null)
            {
                Field field =
                    layout.Fields.Where(f => f.Name.Equals(fldName)).FirstOrDefault();
                if (field != null)
                    return true;
            }

            return false;
        }
        #endregion
    }

    public class Data
    {
        public string ID { get; set; }
        public ComplexData Complex { get; set; }
        public string Bool { get; set; }
    }

    public class ComplexData : INotifyPropertyChanged, IComparable
    {
        private string _name;
        public string Name
        {
            get { return _name; }
            set
            {
                if (_name != value)
                {
                    _name = value;
                    FirePropertyChanged("Name");
                }
            }
        }

        private int _value;
        public int Value
        {
            get { return _value; }
            set
            {
                if (_value != value)
                {
                    _value = value;
                    FirePropertyChanged("Value");
                }
            }
        }



        public event PropertyChangedEventHandler PropertyChanged;

        private void FirePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        public override string ToString()
        {
            return Name;
        }

        public int CompareTo(object obj)
        {
            if (obj is ComplexData)
            {
                Name.CompareTo((obj as ComplexData).Name);
            }
            return -1;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            if (obj is ComplexData)
            {
                return Name.Equals((obj as ComplexData).Value);
            }
            return false;
        }
    }
}
